library(ggplot2)
setwd("Q:/SeventhPlan/RPM - Model/Technical Appendix/")
load <- read.csv("RPMLoadForecast.csv", header=TRUE, stringsAsFactors=FALSE)
load$Year <- c(0, rep(1:19, each=4), rep(20, 3))

#Fit the high and low forecast and take the resulting fit out
high.mod <- lm(log(High/Med)~Year+I(Year^2), data=load)
low.mod <- lm(log(Low/Med)~Year+I(Year^2), data=load)
high.coef <- high.mod$coef
low.coef <- low.mod$coef
high.rsq <- summary(high.mod)$r.squared 
low.rsq <- summary(low.mod)$r.squared 

dev.new()
load.long <- reshape(load, idvar=c("Period", "Year"), varying=list(2:4), times=c("Med", "Low", "High"), direction="long")
load.plot <- ggplot(load.long, aes(x=Period, y=Med, group=factor(time), colour=factor(time))) + geom_line(lwd=1.2)
load.plot <- load.plot + ylab("Load (aMW)") + theme(legend.title=element_blank())
load.plot

load$HighFact <- load$High/load$Med
load$LowFact <- load$Low/load$Med
loadfact.long <- reshape(load[,c("Period", "HighFact", "LowFact")], idvar="Period", varying=list(2:3), times=c("High Factor", "Low Factor"), direction="long")
dev.new()
loadfact.plot <- ggplot(loadfact.long, aes(x=Period, y=HighFact, group=factor(time), colour=factor(time))) + geom_line(lwd=1.2)
loadfact.plot <- loadfact.plot + ylab("Load Factor") + theme(legend.title=element_blank())
loadfact.plot


#Assuming log-symmetry weight the average of the two??
rsq.weight <- c(high.rsq, low.rsq)/(high.rsq + low.rsq)
coef.weight <- high.coef * rsq.weight[1] - low.coef * rsq.weight[2]

#According to Massoud we're trying to hit the high and low forecasts at 85% and 15% respectively
generate.games <- function(x){
  year <- c(0, rep(1:19, each=4), rep(20, 3)) 
  (coef.weight/qnorm(.85)*rnorm(3))%*% t(matrix(c(rep(1,80), year, year^2), nrow=80, ncol=3))
}
games <- sapply(1:800, generate.games)
games.df <- as.data.frame(exp(games)*load$Med)
games.df$period <- 1:80

hist <- read.csv("WNLoadHist.csv", header=TRUE)
hist.long <- reshape(hist, idvar=c("Year", "Type"), varying=list(3:14), direction="long")
hist.long$Quarter <- floor((hist.long$time-1)/3 + 1)
qhist <- aggregate(X1 ~ Quarter + Year + Type, data=hist.long, FUN=mean)
ahist <- aggregate(X1 ~ Year + Type, data=hist.long, FUN=mean)
names(ahist) <- c("Year", "Type", "AnnualAvg")
qhist <- merge(qhist, ahist)
qhist$Factor <- qhist$X1/qhist$AnnualAvg
qhistmean <- aggregate(Factor ~ Quarter + Type, qhist, mean)
names(qhistmean) <- c("Quarter", "Type", "AvgQFactor")
qhist <- merge(qhist, qhistmean)
qhist$NormalizedFactor <- qhist$Factor/qhist$AvgQFactor 
qhist$LogNormalizedFactor <- log(qhist$NormalizedFactor)
qsd <- aggregate(LogNormalizedFactor ~ Quarter + Type, data=qhist, sd)
qhist$Year <- factor(qhist$Year)
qhist.plot <- ggplot(subset(qhist, Type=="Average"), aes(x=Quarter, y=Factor, group=Year, colour=Year)) + geom_line() + geom_point()
qhist.plot 

#Here is seasonal SD beyond annual shapes as applied above
qsd$LogNormalizedFactor[qsd$Type=="Average"]
generate.seasonal <- function(x, q.sds) {
  exp(rnorm(n=80, mean=rep(0, 4), sd=q.sds[c(4,1,2,3)]))
}
seasonal <- sapply(1:800, generate.seasonal, q.sds=qsd$LogNormalizedFactor)
games.df <- games.df*seasonal

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simulation")) + geom_line()
game.plot <- game.plot + geom_line(data=load, aes(x=Period, y=Low, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=load, aes(x=Period, y=High, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Load (aMW)") + theme(legend.title=element_blank())
dev.new()
game.plot

sum(games.df[80, -c(801)] > load$High[80])
sum(games.df[80, -c(801)] > load$High[80])/800

#IF that looks good then here are the parameters
coef.weight/qnorm(.85) * c(1, 20, 20^2)

#Now to estimate the HLH and LLH ratios
hrload <- read.csv("HourlyLoadHist.csv", header=TRUE, stringsAsFactors=FALSE)
hrload.long <- reshape(hrload, idvar=c("Month", "Day", "Hour"), times=1995:2013, varying=list(4:22), direction="long")
names(hrload.long) <- c("Month", "Day", "Hour", "Year", "Load")
hrload.long$HLH <- !(format(hrload.long$TS, "%a") == "Sun" | hrload.long$Hour %in% c(1:6, 23:24))
hrload.long$Quarter <- floor((hrload.long$Month-1)/3 + 1)
qhlh <- aggregate(Load ~ Year + Quarter + HLH, data=hrload.long, FUN=mean)
qload <- aggregate(Load ~ Year + Quarter, data=hrload.long, FUN=mean)
names(qload) <- c("Year", "Quarter", "AvgLoad")
qhlh <- merge(qhlh, qload)
qhlh$Mult <- qhlh$Load/qhlh$AvgLoad

hrwn <- read.csv("HourlyWNLoad.csv")
hrwn$Date <- as.Date(hrwn$Date)
hrwn.long <- reshape(hrwn, idvar="Date", times=1:24, varying=list(2:25), direction="long")
names(hrwn.long) <- c("Date", "Hour", "WN")
hrwn.long$HLH <- !(format(hrwn.long$Date, "%a") == "Sun" | hrwn.long$Hour %in% c(1:6, 23:24))
hrwn.long$Quarter <- floor((as.numeric(format(hrwn.long$Date, "%m"))-1)/3 + 1)
hrwn.long$Year <- as.numeric(format(hrwn.long$Date, "%Y"))
qwnhlh <- aggregate(WN ~ Year + Quarter + HLH, data=hrwn.long, FUN=mean)
qwn <- aggregate(WN ~ Year + Quarter, data=hrwn.long, FUN=mean)
names(qwn) <- c("Year", "Quarter", "AvgWN")
qwnhlh <- merge(qwnhlh, qwn)
qwnhlh$Mult <- qwnhlh$WN/qwnhlh$AvgWN
qwnhlh.wide <- reshape(qwnhlh[qwnhlh$Year >= 2015,], idvar=c("Year", "Quarter"), timevar="HLH", direction="wide")
qwnhlh.wide[, c("Year", "Quarter", "Mult.FALSE", "Mult.TRUE")]

#See if the old parameter looks similar
#old.param <- sapply(1:800, function(x){.06/20*rnorm(1)*c(0, rep(1:19, each=4), rep(20, 3))})
#old.seas <- sapply(1:800, function(x){exp(rnorm(n=80, mean=rep(0,4), sd=c(0, .05, 0, .05) ))})
#old.param.df <- as.data.frame(exp(old.param)*load$Med)
#old.param.df <- old.param.df*old.seas
#old.param.df$period <- 1:80
#long.old.df <- reshape(old.param.df, idvar="period", varying=list(1:800), direction="long")
#old.plot <- ggplot(long.old.df, aes(x=period, y=V1, group=time)) + geom_line()
#old.plot <- old.plot + geom_line(data=load, aes(x=Period, y=Low, group="LOW"), colour="blue", lwd=1.2)
#old.plot <- old.plot + geom_line(data=load, aes(x=Period, y=High, group="HIGH"), colour="red", lwd=1.2)
#dev.new()
#old.plot
#
#sum(old.param.df[80,-c(801)]> load$High[80])
#sum(old.param.df[80,-c(801)]> load$High[80])/800
