library(ggplot2)
setwd("Q:/SeventhPlan/RPM - Model/Technical Appendix/")
peak <- read.csv("RPMPeakRatioForecast.csv", header=TRUE, stringsAsFactors=FALSE)
peak$Year <- c(0, rep(1:19, each=4), rep(20, 3))

#Fit the high and low forecast and take the resulting fit out
high.mod <- lm(log(High/Med)~Year+I(Year^2), data=peak)
low.mod <- lm(log(Low/Med)~Year+I(Year^2), data=peak)
high.coef <- high.mod$coef
low.coef <- low.mod$coef
high.rsq <- summary(high.mod)$r.squared 
low.rsq <- summary(low.mod)$r.squared 

peak.long <- reshape(peak, idvar=c("Period", "Year"), varying=list(2:4), times=c("Med", "Low", "High"), direction="long")
peak.plot <- ggplot(peak.long, aes(x=Period, y=Med, group=factor(time), colour=factor(time))) + geom_line(lwd=1.2)
peak.plot <- peak.plot + ylab("Load (aMW)") + theme(legend.title=element_blank())
dev.new()
peak.plot

#Fitting the high coefficients to focus on that tail
rsq.weight <- c(high.rsq, low.rsq)/(high.rsq + low.rsq)
coef.weight <- high.coef * 1 - low.coef * 0

#According to Massoud we're trying to hit the high and low forecasts at 85% and 15% respectively
generate.games <- function(x){
  year <- c(0, rep(1:19, each=4), rep(20, 3)) 
  (coef.weight/qnorm(.85)*rnorm(3))%*% t(matrix(c(rep(1,80), year, year^2), nrow=80, ncol=3))
}
games <- sapply(1:800, generate.games)
games.df <- as.data.frame(exp(games)*peak$Med)
games.df$period <- 1:80

hist <- read.csv("WNLoadHist.csv", header=TRUE)
hist.long <- reshape(hist, idvar=c("Year", "Type"), varying=list(3:14), direction="long")
hist.long$Quarter <- floor((hist.long$time-1)/3 + 1)
qhist <- aggregate(X1 ~ Quarter + Year + Type, data=hist.long, FUN=mean)
ahist <- aggregate(X1 ~ Year + Type, data=hist.long, FUN=mean)
names(ahist) <- c("Year", "Type", "AnnualAvg")
qhist <- merge(qhist, ahist)
qhist$Factor <- qhist$X1/qhist$AnnualAvg
qhistmean <- aggregate(Factor ~ Quarter + Type, qhist, mean)
names(qhistmean) <- c("Quarter", "Type", "AvgQFactor")
qhist <- merge(qhist, qhistmean)
qhist$NormalizedFactor <- qhist$Factor/qhist$AvgQFactor 
qhist$LogNormalizedFactor <- log(qhist$NormalizedFactor)
qsd <- aggregate(LogNormalizedFactor ~ Quarter + Type, data=qhist, sd)
qhist$Year <- factor(qhist$Year)
qhist.plot <- ggplot(subset(qhist, Type=="Max"), aes(x=Quarter, y=NormalizedFactor, group=Year, colour=Year)) + geom_line() + geom_point()
dev.new()
qhist.plot 

#Here is seasonal SD beyond annual shapes as applied above
qsd$LogNormalizedFactor[qsd$Type=="Max"]
generate.seasonal <- function(x, q.sds) {
  exp(rnorm(n=80, mean=rep(0, 4), sd=q.sds[c(4,1,2,3)]))
}
seasonal <- sapply(1:800, generate.seasonal, q.sds=qsd$LogNormalizedFactor[qsd$Type=="Max"])
games.df <- games.df*seasonal

long.game.df <- reshape(games.df, idvar="period", varying=list(1:800), direction="long")
game.plot <- ggplot(subset(long.game.df, time%in%1:10), aes(x=period, y=V1, group=time, colour="Simulation")) + geom_line()
game.plot <- game.plot + geom_line(data=peak, aes(x=Period, y=Low, group="LOW", colour="Low"), lwd=1)
game.plot <- game.plot + geom_line(data=peak, aes(x=Period, y=High, group="HIGH", colour="High"), lwd=1)
game.plot <- game.plot + ylab("Peak Load (MW)") + theme(legend.title=element_blank())
dev.new()
game.plot

sum(games.df[80, -c(801)] > peak$High[80])
sum(games.df[80, -c(801)] > peak$High[80])/800

#IF that looks good then here are the parameters
coef.weight/qnorm(.85) * c(1, 20, 20^2)

